%   J_CoM
%
%   The function computes the Jacobian of the Center of Mass of the robot
%   describing the (linear) velocity of the CoM in absolute frame
%
%   Syntax: J_com = J_CoM(P, Z, g_st0, q, Cc, m, (opt.)joint_type )
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   g_m_0 \in R^{4 X 4 X number of masses (or joints) } = collection of
%       homogenous transformation matrices, one per each link or per each mass,
%       in the -zero- configuration of the robot. The
%       trasformation for zero-mass links can be omitted
%   q \in R^{1 X (numb. of joints) } = robot configuration used for the
%              computation
%   Cc \in R^{(numb. of masses) X (numb. of joints) } = each row describes
%        a serial kinematic chain of the tree;
%        the i-th element of the row is 1 (true) if the i-th joint belongs to
%        the chain, 0 (false) otherwise
%   m  \in R^(numb. of joints) mass of each heavy element of the robot
%       it is supposed to have one mass for each link, at lest one element
%       of the vector has to be non-null; m(i) it the mass of the link
%       attached to the i-th joint
%   joint_type \in R^{1 X (numb. of joints) } = OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%           if joint_type is empty, all the joints are considered revolute, if it
%           is a scalar, all the joints are of the same type
%
%   Outputs:
%   J_com \in R^{(3*numb. of chians) X (numb. of joints) } = Jacobian matrix of the Center of Mass of the robot
%   describing the velocity of the CoM in absolute frame

function plot_Masses(P_0, Z_0, g_m_0, q_actual, Cc, m, varargin )
%
if(size(varargin,1)~=0)
    joint_type = varargin{1} ;
else
    joint_type = zeros(1,size(Z_0,2)) ;
end
if(length(joint_type)==1)
    joint_type = joint_type*ones(1,size(Z_0,2));
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cont = 1;
g_m_temp = zeros(4,4, length( m)) ;
if ( size(g_m_0,3)~= length(m) )
    for i = 1: length(m)
        if(m(i)==0)
            continue ;
        else
            g_m_temp(:,:,i) = g_m_0(:,:,cont) ;
            cont = cont + 1 ;
        end
    end
end
g_m_0 = g_m_temp ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
num_chains = size(Cc,1) ;
for i = 1 : num_chains
    chain_i = find(Cc(i,:)) ; %varargin{i};
    P_chain = P_0(:,chain_i) ;
    Z_chain = Z_0(:,chain_i) ;
    g_m_chain = g_m_0(:,:,chain_i) ;
    q_actual_chain = q_actual(chain_i) ;
    joint_type_chain = joint_type(chain_i ) ;
    m_chain = m(chain_i )  ;
    %
    for k = 1 : length(chain_i)
%         P_ = [  P_   P(:,chain_i(k))  ] ;
%         Z_ = [  Z_   Z(:,chain_i(k))  ] ;
%         q_ = [  q_   q(chain_i(k))    ] ;
%         joint_type_ = [joint_type_   joint_type(chain_i(k))] ;
%         m_ = [ m_ q(chain_i(k)) ] ; 
        if(m_chain(k)~=0)
            P_plot = P_chain(:,1:k) ;
            Z_plot = Z_chain(:,1:k) ;
            g_m_plot = g_m_chain(:,:,k) ;
            q_plot = q_actual_chain(1:k) ;
            joint_type_plot = joint_type_chain(1:k) ;
            %m_plot = m_chain(k) ;
            g_m_plot_final = fw_kin_serial(P_plot,Z_plot,g_m_plot,q_plot,joint_type_plot) ;
            plot_Object(g_m_plot_final, .05,0,'Sphere') ;
        end
    end
end

% plot_Object(Rotrasl,radius,height,type)


% 
% 
% if isempty(findobj(gcf,'type','axes'))
%     ax = axes;
%     grid on;
%     axis equal;
%     %camlight left;
%     set(ax,'XLimMode','auto');
%     set(ax,'YLimMode','auto');
%     set(ax,'ZLimMode','auto');
% else
%     ax = gca;
% end
% 
% switch(type)
% 
%     case 'Sphere'
%         
%         [x1,y1,z1]=sphere(60);
%         h = surface(x1*radius,y1*radius,z1*radius,'FaceColor','k');
%         set(h,'EdgeColor','none');
%         hs = hgtransform('Parent',ax);
%         set(h,'Parent',hs);
%         set(hs,'Matrix',Rotrasl);
%     
% 
% 
%     case 'Cylinder'
% 
%         X = circle([0 0 0],radius,'n',100);
% 
%         h = hgtransform('Parent',ax);
% 
%         % cylinder body
%         [x1,y1,z1]=cylinder(radius,60);
%         h1 = surface(x1,y1,(z1*height),'FaceColor','r');
%         set(h1,'EdgeColor','none');
%         hs1 = hgtransform('Parent',h);
%         T1 = makehgtform('translate',[0,0,-(height)/2]);
%         set(h1,'Parent',hs1);
%         set(hs1,'Matrix',T1);
%         set(hs1,'Parent',h);
% 
%         % cylinder base
%         h2 = patch(X(1,:)',X(2,:)',X(3,:)','FaceColor','r');
%         set(h2,'EdgeColor','none');
%         hs2 = hgtransform('Parent',h);
%         T2 = makehgtform('translate',[0,0,-(height)/2]);
%         set(h2,'Parent',hs2);
%         set(hs2,'Matrix',T2);
%         set(hs2,'Parent',h);
% 
%         % cylinder top
%         h3 = patch(X(1,:)',X(2,:)',X(3,:)','FaceColor','r');
%         set(h3,'EdgeColor','none');
%         hs3 = hgtransform('Parent',h);
%         T3 = makehgtform('translate',[0,0,(height)/2]);
%         set(h3,'Parent',hs3);
%         set(hs3,'Matrix',T3);
%         set(hs3,'Parent',h);
%         
%         set(h,'Matrix',Rotrasl);
% 
%     otherwise
%         error('MANIPULATEDOBJECT: no plot metod defined for ''%s''', mo.name);
%             
% end



end


