% forward_kinematics
%
% Evaluates Forward Kinematics for a serial chain; this function requires
% the user for the configuration in which will be calculated the
% kinematics.
%
%   Inputs:
%   P = Joint origins coordinates
%   Z = Joint axis
%   g_st0 = trasformation matrix from base to end-effector frame
%   q0 = Joint configuration in which will be evaluated the kinematics(degrees)
%   the 5-th optional input is a 1/0 vector containing which are the prismatic joints
%   1 for prismatic joints, 0 for revolute joints  
%
%   Outputs:
%   T = trasformation matrix from Base to End-Effector frame
%
%
%   Usage Example: T = forward_kinematics(P,Z,g_st0,[.1 .2 .4 .9], [1 0 0 1 ])
%
%   Means that we want to evaluate the kinematics for a 4-joint chain, of
%   which the 1st and the 4th are prismatic joints.
%

function T = forward_kinematics_NOSYM(P,Z,g_st0,q0,varargin)

for i = 1 : size(Z,2)
    Z(:,i) = unit(Z(:,i));
end
qd = size(Z,2);  % takes the joint number from the joint axis
joints = zeros(1,qd);    % joints vector: 0 for rotational, 1 for prismatic joints


if(nargin == 5) % checks if there is an input containing the prismatic joints configuration
    vett = varargin{1};
    for i = 1 : size(vett,2)
        joints(i) = vett(i);
    end
end

for i = 1 : qd   % for all the joints
    
    if(joints(i) == 1) % prismatic
        xi_i = [Z(:,i); zeros(size(Z(:,i),1),1)];   % twist
        
    else % revolute
        xi_i = [-cross(Z(:,i),P(:,i)) ; Z(:,i)];    % twist
    end
    g_i = twistexp2(xi_i, q0(i));
    if(i == 1)
        T = g_i;
    else
        T = T*g_i;
    end

end

T = T*g_st0;

end