% fw_kin_serial : 
%
% Evaluates Forward Kinematics for a serial chain.
%   
%   Syntax: T = fc_kin_serial( P, Z, g_st0, q, (opt.)joint_type )
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   g_st0 \in R^{4 X 4 } = homogenous trasformation matrix from base to end-effector frame 
%   q \in R^{1 X (numb. of joints) } = 
%           Joint configuration in which will be evaluated the forward kinematics 
%   joint_type \in R^{1 X (numb. of joints) } =  OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%   if joint_type is empty, all the joints are considered revolute, if it
%   is a scalar, all the joints are of the same type
%
%   Outputs:
%   T \in R^{4 X 4 } = trasformation matrix from Base to End-Effector frame, in the
%   indicated configuration
%
%   Usage Example: T = fw_kin_serial(P,Z,g_st0,[.1 .2 .4 .9], [1 0 0 1 ])
%
%   Means that we want to evaluate the kinematics for a 4-joint chain, in
%   which the 1st and the 4th are joints are prismatic.
%

function T = fw_kin_serial( P , Z , g_st0 , q , varargin ) 

for i = 1 : size(Z,2)
    Z(:,i) = unit(Z(:,i));
end
qd = size(Z,2);  % takes the joint number from the joint axis
joints = zeros(1,qd);    % joints vector: 0 for rotational, 1 for prismatic joints

if(nargin >= 5) % checks if there is an input containing the prismatic joints configuration
    vett = varargin{1};
    for i = 1 : size(vett,2)
        joints(i) = vett(i);
    end
    if (size(vett,2)==1)
    joints = vett*ones(1,size(Z,2));
    end
end

for i = 1 : qd   % for all the joints
    
    if(joints(i) ~= 0 ) % if it's a prismatic joint
        xi_i = [Z(:,i); zeros(size(Z(:,i),1),1)];   % twist
        
    else % if it's a revolute joint
        xi_i = [-cross(Z(:,i),P(:,i)) ; Z(:,i)];    % twist
    end
    
    g_i = twistexp2(xi_i, q(i));
    if(i == 1)
        T = g_i;
    else
        T = T*g_i;
    end

end

T = T*g_st0;

end

