%   Body_Jacobian_tree
%
%   The function calculates the body Jacobian for a serial kinematic chain
%   in the given configuration
%
%   Syntax: Body_J_tree = Body_Jacobian_tree(P, Z, g_st0, q, Cc, (opt.)joint_type )
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   g_st0 \in R^{4 X 4 } = homogenous trasformation matrix from base to end-effector frame
%   q \in R^{1 X (numb. of joints) } = robot configuration used for the
%              computation
%   Cc \in R^{(numb. of chains) X (numb. of joints) } = each row describes
%        a serial kinematic chain of the tree; 
%        the i-th element of the row is 1 (true) if the i-th joint belongs to
%        the chain, 0 (false) otherwise
%   joint_type \in R^{1 X (numb. of joints) } = OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%   if joint_type is empty, all the joints are considered revolute, if it
%   is a scalar, all the joints are of the same type
%
%   Outputs:
%   Body_J_tree \in R^{(6*numb. of chians) X (numb. of joints) } = Body
%   Jacobian of the kinematic tree
%
function Body_J_tree = Body_Jacobian_tree(P, Z, g_st0, q, Cc, varargin ) %
%
joint_type = zeros(1,size(Z,2)) ;
if(size(varargin,1)~=0) %if(nargin > 5) %
    vett = varargin{1};
    if (length(vett)==1)
        joint_type = vett*ones(1,size(Z,2)) ;
    else
        for i = 1 : length(vett)
            joint_type(i) = vett(i);
        end
    end
end
%
n_ee = size(Cc,1) ;
Body_J_tree = zeros(6*n_ee, size(Z,2)); % initialization
for k = 1 : n_ee
    %
    Cc_serial = Cc(k,:) ;
    Body_J_serial_i = zeros(6 , size(Z,2)) ;
    g_next = eye(4) ;
    %
    for i = 1 : size(Z,2)
        if(i==1) % the screw for the first joint
            if(joint_type(i)==0)  % if revolute joint
                xi_i = [-cross(unit(Z(:,i)), P(:,i)) ; unit(Z(:, i))] * logical(Cc_serial(i)) ; % revolute
                Body_J_serial_i(:,i) = xi_i ;
                g_next = twistexp2(xi_i,q(i));
            else  % if prismatic joint
                xi_i = [unit(Z(:,i)); zeros(3,1)] * logical(Cc_serial(i)); % prismatic
                Body_J_serial_i(:,i) = xi_i ;
                g_next = twistexp2(xi_i,q(i));
            end
        else  % the screw for the joints from the 2nd to the last
            if(joint_type(i)==0)  % if revolute joint
                xi_i = [-cross(unit(Z(:,i)),P(:,i)) ; unit(Z(:,i))] * logical(Cc_serial(i)); % revolut
                Body_J_serial_i(:,i) = ad2(g_next)*xi_i ; %
                g_next = g_next*twistexp2(xi_i,q(i));
            else  % if prismatic joint
                xi_i = [unit(Z(:,i)); zeros(3,1)] * logical(Cc_serial(i)); % prismatic
                Body_J_serial_i(:,i) = ad2(g_next)*xi_i ; %
                g_next = g_next*twistexp2(xi_i,q(i));
            end
        end
    end
    % 
    Body_J_tree((1+(k-1)*6) :(6+(k-1)*6),:) = ad2(ihomogeneous(g_next*g_st0(:,:,k)))* Body_J_serial_i ;
% 
end
%
end





