%    singleSerialJacobian
%
%    The function calculates the spatial Jacobian for the serial chain
%    input configuration, using twist notation.
%
%    Usage: J_chain_i = singleSerialJacobian(P,Z,q,conf,joints)
%
%    Inputs:
%    P = matrix whose i-th column is the axis point for the i-th joint
%    Z = matrix whose i-th column represent the i-th joint axis.
%    conf = vector (list) containing the joint numbers belonging to the
%           serial chain
%    joint_type = the i-th element is 0 if i-th joint is revolute
%             1 if it is prismatic joint.
%
%    Returns:
%    J_chain_i = the spatial Jacobian for the serial chain
%

function J_chain_i = singleSerialJacobian(P,Z,q,conf,joint_type)
joints = joint_type ;
if(size(P) ~= size(Z))
    error('P and Z must be the same size');
end

if ( (size(q,1) ~= 1) && (size(q,2) ~= 1) )
    error('q must be a [1 x num_joints] vector');
end

n = size(conf,2);  % joints number for the actual serial chain configuration

J_chain_i = [];
for i = 1 : n
    
    c = joints(conf(i));
    switch(c)   % switch between prismatic or revolute joint
        case 0  % revolute joint
        if(i == 1)  % evaluate xi
            xi_i = [-cross(Z(:,conf(i)),P(:,conf(i))) ; Z(:,conf(i))];
            J_chain_i = [xi_i];
            xi_next = twistexp2(xi_i,q(i));
        else  %  evaluate xi'
            xi_i = [-cross(Z(:,conf(i)),P(:,conf(i))) ; Z(:,conf(i))];
            J_chain_i(:,conf(i)) = ad2(xi_next)*xi_i ; %   = [J_chain_i ad2(xi_next)*xi_i];
            xi_next = xi_next*twistexp2(xi_i,q(i));
        end
        case 1  % prismatic joint
        if(i == 1)  % evaluate xi
            xi_i = [Z(:,conf(i)); zeros(size(Z(:,conf(i)),1),1)];
            J_chain_i = [xi_i];
            xi_next = twistexp2(xi_i,q(i));
        else  %  evaluate xi'
            xi_i = [Z(:,conf(i)); zeros(size(Z(:,conf(i)),1),1)];
            J_chain_i(:,conf(i)) = ad2(xi_next)*xi_i ; % = [J_chain_i ad2(xi_next)*xi_i ];
            xi_next = xi_next*twistexp2(xi_i,q(i));
        end
    end          
end

end