%   D_Jt_f_tree
%
%   The function calculates the derivative of J^T*f with respect to q
%   for a kinematic tree
%
%   [ Q_ci ] = D_Jt_f_serial( P, Z, EE_ci_0, q, B_ci, f_ci, cf_dim , (opt.)joint_type )
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   EE_c_0 \in R^{4 X 4 X (numb. of chains)} = Collecnting the 
%       homogeneous transformation describing the pose
%       of the contact frames w.r.t {A} in the -zero- configuration
%   q \in R^{1 X (numb. of joints) } = actual configuration of the robot
%   Cc \in R^{(numb. of chains) X (numb. of joints) } = each row describes
%        a serial kinematic chain of the tree; 
%        the i-th element of the row is 1 (true) if the i-th joint belongs to
%        the chain, 0 (false) otherwise
%   B_ci \in R^{6 X (contact force components) } =  transmittable wrench matrix, its transpose is 
%       a Selection matrix, selecting only the components of the contact twist violating the
%   kinematic contact constraint; B_c = [B_c1, B_c2, ... B_cn] 
%   f_c \in R^{total numb. contact force components } = actual forces
%       exerted on the contact; f_c = [ f_c1 ; f_c2; ... ; f_cn ]
%   cf_dim \in R^{ (numb. of chains)} = the i-th element specifies the number of
%   the contact constraint on the i-th contact point
%   joint_type \in R^{1 X (numb. of joints) } =  OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%           if joint_type is empty, all the joints are considered revolute, if it
%           is a scalar, all the joints are of the same type
%
%   Outputs:
%   Q_ci \in R^{(numb. of joints) X (numb. of joints) } = derivative of J^T*f with respect to q
%
function [ Q_c ] = D_Jt_f_tree( P, Z, EE_c_0, q, Cc, B_c, f_c, cf_dim , varargin  )
%
Q_c = zeros(length(q)) ; % initialization for improving speed
%
joint_type = zeros(1,size(Z,2)) ;
if(nargin >= 9) %
    vett = varargin{1};
    if (length(vett)==1)
        joint_type = vett*ones(1,size(Z,2)) ;
    else
        for i = 1 : length(vett)
            joint_type(i) = vett(i);
        end
    end
end
%
start_i = zeros(size(cf_dim)); % index of start of the fc_i vector
start_i(1) = 1 ;
for j = 2:length(cf_dim)
    start_i(j) = start_i(j-1)+cf_dim(j-1); 
end
end_i = zeros(size(cf_dim)); % index of end of the fc_i vector
for j = 1:length(cf_dim)
    end_i(j) = start_i(j) + cf_dim(j)-1 ;
end
%
%
for i=1:size(EE_c_0,3)    % for the i-th kinematic chain
    Cc_row_i = Cc(i,:);
    P_temp_i = [];
    Z_temp_i = [];
    q_temp_i = [];
    joint_type_temp_i = [];
    for h = 1:length(Cc_row_i)
        if(Cc_row_i(h)~=0)
            P_temp_i(:,end+1) = P(:,h);
            Z_temp_i(:,end+1) = Z(:,h);
            q_temp_i(end+1) = q(h);
            joint_type_temp_i(end+1) = joint_type(h);
        end
    end
    J_spa_temp_ci = Spatial_Jacobian_serial(P_temp_i, Z_temp_i, ...
        q_temp_i, joint_type_temp_i ) ;
    J_spa_ci = zeros(size(J_spa_temp_ci,1), length(Cc_row_i) );
    cont = 1;
    for h = 1:length(Cc_row_i)
        if(Cc_row_i(h)~=0)
            J_spa_ci(:,h) = J_spa_temp_ci(:,cont);
            cont = cont + 1;
        else
            J_spa_ci(:,h) = zeros(size(J_spa_temp_ci,1),1) ;
        end
    end
    g_aci_0 = EE_c_0(:,:,i);
    B_ci = B_c(:,start_i(i):end_i(i)) ;
    f_ci = f_c(start_i(i):end_i(i));
    B_ci_T = B_ci';
    %
    Q_ci = zeros(size(J_spa_ci,2),size(J_spa_ci,2));
    for j= 1:size(J_spa_ci,2) % derivative w.r.t. the j-th joint variable
        Q_ci(:,j) = (( D_Jacob_spa_i(J_spa_ci,j))'*(ad2(ihomogeneous(g_aci_0)))'...
            * B_ci_T'*f_ci ...
            +...
            J_spa_ci'* ((ad2(ihomogeneous(g_aci_0))*ad_lie(-J_spa_ci(:,j) ) )' )...
            *B_ci_T'*f_ci) ;
    end
    Q_c = Q_c + Q_ci ;
    %
end
%
end




