%   J_CoM
%
%   The function computes the Jacobian of the Center of Mass of the robot
%   describing the (linear) velocity of the CoM in absolute frame
%
%   Syntax: J_com = J_CoM(P, Z, g_st0, q, Cc, m, (opt.)joint_type )
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   g_m_0 \in R^{4 X 4 X number of masses (or joints) } = collection of
%       homogenous transformation matrices, one per each link or per each mass,
%       in the -zero- configuration of the robot. The
%       trasformation for zero-mass links can be omitted
%   q \in R^{1 X (numb. of joints) } = robot configuration used for the
%              computation
%   Cc \in R^{(numb. of masses) X (numb. of joints) } = each row describes
%        a serial kinematic chain of the tree;
%        the i-th element of the row is 1 (true) if the i-th joint belongs to
%        the chain, 0 (false) otherwise
%   m  \in R^(numb. of joints) mass of each heavy element of the robot
%       it is supposed to have one mass for each link, at lest one element
%       of the vector has to be non-null; m(i) it the mass of the link
%       attached to the i-th joint
%   joint_type \in R^{1 X (numb. of joints) } = OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%           if joint_type is empty, all the joints are considered revolute, if it
%           is a scalar, all the joints are of the same type
%
%   Outputs:
%   J_com \in R^{(3*numb. of chians) X (numb. of joints) } = Jacobian matrix of the Center of Mass of the robot
%   describing the velocity of the CoM in absolute frame
%
function J_com = J_CoM(P, Z, g_m_0, q, Cc, m, varargin ) %
%
joint_type = zeros(1,size(Z,2)) ;
if(nargin >= 7) %
    vett = varargin{1};
    if (length(vett)==1)
        joint_type = vett*ones(1,size(Z,2)) ;
    else
        for i = 1 : length(vett)
            joint_type(i) = vett(i);
        end
    end
end
%
cont = 1;
g_m_temp = zeros(4,4, length( m)) ;
if ( size(g_m_0,3)~= length(m) )
    for i = 1: length(m)
        if(m(i)==0)
            continue ;
        else
            g_m_temp(:,:,i) = g_m_0(:,:,cont) ;
            cont = cont + 1 ;
        end
    end
end
g_m_0 = g_m_temp ;
%
M = sum(m) ;
%
J_com = zeros(6, size(P,2 ));
n_ee = size(Cc,1) ; % number of end-effectors
for k = 1 : size(Z,2)
    if(m(k)~=0)
        for l = 1:n_ee
            if(Cc(l,k)~=0)
                Cc_temp = Cc(l,:) ;
                break ;
            else
                continue ;
            end
        end
        % P, Z, g_m_0, q, Cc
        index_temp = find(find(Cc_temp)<=k) ;
        P_temp = P(:,index_temp) ; %  P(:,find(Cc_temp)) ;
        Z_temp = Z(:,index_temp) ;
        q_temp = q(index_temp) ;
        joint_type_temp = joint_type(index_temp) ;
        J_m_k = Spatial_Jacobian_serial(P_temp, Z_temp, q_temp, joint_type_temp) ;
        J_m_k_long = zeros(6, size(Cc,2)) ;
        cont_2 = 1;
        for f=1:size(Cc,2)
            if (Cc_temp(f)~=0&&(f<=k) )
                g_m_temp = fw_kin_serial( P_temp,...
                    Z_temp , g_m_0(:,:,k) , q_temp , joint_type_temp ) ;
                %g_m_0(:,:,k) ;%P(:,find(Cc_temp)) ;
                J_m_k_long(:,f) = J_m_k(:,cont_2) ;
                cont_2 = cont_2 +1 ;
            else
                continue
            end
        end
        J_com = J_com + m(k)*ad2(ihomogeneous( eye(3), g_m_temp(1:3,4)))* J_m_k_long ;
    else
        continue
    end
end
J_com = (1/M)*J_com ;
J_com = J_com(1:3,:) ;
%
end





