%   Stance_Matrix
%
%   The function calculates the body Jacobian for a serial kinematic chain
%   in the given configuration
%
%   Syntax: Stance_M = Stance_Matrix(P, Z, g_st0, q, Cc, vkc_dim , (optional)joint_type ) 
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   g_st0 \in R^{4 X 4 } = homogenous trasformation matrix from base to end-effector frame
%   q \in R^{1 X (numb. of joints) } =
%           Joint configuration in which will be evaluated the forward kinematics
%   Cc \in R^{(numb. of chains) X (numb. of joints) } = each row describes
%        a serial kinematic chain of the tree; 
%        the i-th element of the row is 1 (true) if the i-th joint belongs to
%        the chain, 0 (false) otherwise
%   vkc_dim = number of joints belonging to the Virtual Kinematic Chain
%   joint_type \in R^{1 X (numb. of joints) } = OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%   if joint_type is empty, all the joints are considered revolute, if it
%   is a scalar, all the joints are of the same type
%
%   Outputs:
%   Stance_M \in R^{(numb. of joints of the VKC) X (6*numb. of chians)} = Stance Matrix
%
function Stance_M = Stance_Matrix(P, Z, g_st0, q, Cc, vkc_dim , B_c, cf_dim, varargin ) %
%
joint_type = zeros(1,size(Z,2)) ;
if(size(varargin,1)~=0) %
    vett = varargin{1};
    if (length(vett)==1)
        joint_type = vett*ones(1,size(Z,2)) ;
    else
        for i = 1 : length(vett)
            joint_type(i) = vett(i);
        end
    end
end
%
Body_J_tree = Body_Jacobian_tree(P, Z, g_st0, q, Cc, joint_type ) ;
Stance_M_T_temp  = Body_J_tree(:,1:vkc_dim) ;
%
Stance_M_T = zeros(sum(cf_dim),size(Stance_M_T_temp,2)) ;
for i = 1: size(Cc,1)
   cf_temp = cf_dim ;
   cf_temp((i+1):end)  = 0 ;
   last_value_cf = cf_temp( max(find(cf_temp)) ) ;
   last_B_temp   = sum(cf_temp) ; 
   first_B_temp  = last_B_temp - last_value_cf + 1 ; 
   %
   J_temp = Stance_M_T_temp((6*(i-1)+1):6*i,:) ;
   B_temp = B_c(:, first_B_temp :last_B_temp )
   Bt_J = B_temp'*J_temp ;
   Stance_M_T(first_B_temp : last_B_temp , :) = Bt_J ;
end
%
Stance_M = Stance_M_T' ;
%
end






















