% Ru_elastic
%
% Provide the map between joint reference displacement (motor displacement) and floating base displacement (VKC joint displacemnt),
% such that d_u = -Ru d_q_r
% for elastic humanoid robot
%
%   Syntax:  [ Ru ] = Ru_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, K_q )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%   K_q \in R^{(numb. of joints) X (numb. of joints)} = 
%               Joint stiffness matrix (usually it is diagonal) %
%
%   Outputs:
%   Ru_elastic is the map between joint reference displacement (motor displacement) and floating base displacement (VKC joint displacemnt),
% such that d_u = -Ru d_q_r
% for elastic humanoid robot
%
function [ Ru ] = Ru_elastic( J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, K_q, varargin )
%
if(size(varargin,1)~=0)
    tol = varargin{1} ;
else
    tol = 1E0;
end
%
s_w = size(S_c,1) ; 
s_q  = size(J_c,2) ; 
s_fc  = size(J_c,1) ; 
%
cFLMM  = cFLMM_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c,  K_q );
Ru_dirty = cFLMM( s_fc+s_q+1 : s_fc+s_q+s_w , end-s_q+1:end ) ;
% Ru_dirty_temp = Ru_dirty ;
% Ru_dirty_temp(~Ru_dirty_temp)=inf ;
% minimum = min(min(abs(Ru_dirty_temp))) ; 
Ru = Ru_dirty ; %round(Ru_dirty./(minimum*tol))*minimum*tol ;
end


