% Rf_rigid
%
% Provide the map between joint displacement and contact force variation,
% such that d_fc = -Rf d_q
% for rigid humanoid robot
%
%   Syntax:  [ Rf ] = Rf_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%
% Outputs:
% Rf_rigid is the map between joint displacement and contact force variation,
% such that d_fc = -Rf d_q for rigid humanoid robot
%
function [ Rf ] = Rf_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, varargin )
%
%
if(size(varargin,1)~=0) %
    tol = varargin{1} ;
else
    tol = 1E-2 ; %  
end
%
s_fc  = size(J_c,1) ; 
s_q  = size(J_c,2) ; 
%
cFLMM  = cFLMM_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c );
R_f_dirty = cFLMM(1:s_fc, end-s_q+1:end) ;
R_f_temp = R_f_dirty ;
R_f_temp(~R_f_temp)=inf ;
minimum = min(min(abs(R_f_temp))) ; 
Rf =round(R_f_dirty./(minimum*tol))*minimum*tol ;  % R_f_dirty ; % 
%Rf = fix(R_f_temp./(10.^(floor(log10(abs(R_f_temp)))))) .*(10.^(floor(log10(abs(R_f_temp))))) ;
%
end