% fw_kin_tree :
%
% Evaluates Forward Kinematics for a tree-structured kinematic chain.
%   
%   Syntax: T = fw_kin_tree( P, Z, g_st0, q, Cc, (opt.)joint_type )
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecting joint axis vectors
%        in the -zero- configuration of the robot
%   g_st_tree \in R^{4 X 4 X (numb. of chains) } = homogenous transformation matrices
%            from base to all the end-effector frames in the -zero-
%            configuration of the robot
%   q \in R^{1 X (numb. of joints) } = 
%           Joint configuration in which will be evaluated the forward kinematics 
%   Cc \in R^{(numb. of chains) X (numb. of joints) } = each row describes
%        a serial kinematic chain of the tree; 
%        the i-th element of the row is 1 (true) if the i-th joint belongs to
%        the chain, 0 (false) otherwise
%   joint_type \in R^{1 X (numb. of joints) } =  OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%   if joint_type is empty, all the joints are considered revolute, if it
%   is a scalar, all the joints are of the same type
%
%   Outputs:
%   T \in R^{4 X 4 X (numb. of chains)} = homogenous trasformation matrices
%            from base to all the end-effector frames in the given
%            configuration of the robot
%

function T_tree = fw_kin_tree(P,Z, g_st_tree, q, Cc, varargin)
%
% Checking if there is an input containing the joint type description
% If it's not there, the joints are assumed to be revolute
joint_type = zeros(1,size(Z,2)) ;
if(nargin >= 6) %
    vett = varargin{1};
    if (length(vett)==1)
        joint_type = vett*ones(1,size(Z,2));
    else
        for i = 1 : length(vett)
            joint_type(i) = vett(i);
        end
    end
end
%
T_tree = g_st_tree ;  %Initialization of the output, for efficiency
%
for i = 1 : size(g_st_tree,3)  % for each end effector ...
    connect_serial = Cc(i,:);  % Chosing the chain descriptor
    P_serial = [] ;
    Z_serial = [] ;
    q_serial = [] ;
    joints_serial = [] ;
    % Building the matrices for the serial chian
    for j = 1 : length(connect_serial)
        if (connect_serial(j)~=0)
            P_serial(:, end+1) = P(:,j) ;
            Z_serial(:, end+1)  = Z(:,j) ;
            q_serial(end+1) = q(j) ;
            joints_serial(end+1) = joint_type(j) ;
        end
    end
    %
    T_tree(:,:,i) = fw_kin_serial(P_serial, Z_serial, g_st_tree(:,:,i),...
        q_serial , joints_serial ) ;
    %
end
%
end



