%    spatial_WB_Jacobian
%
%    The function calculates the Whole-Body spatial Jacobian for the given
%    input configuration, using twist notation.
%
%    Usage: J = spatial_WB_Jacobian(P,Z,joints,varargin)
%
%    Inputs:
%    P = matrix whose i-th column is the axis point for the i-th joint
%    Z = matrix whose i-th column represent the i-th joint axis. Otherwise,
%        Z can be only a vector, because in most cases all the joints have 
%        the same joint axis.
%    joints = vector of boolean elements with dimensions [1 x num_joints],
%             the i-th element is 0 if the i-th joint is revolute,
%             the i-th element is 1 if the i-th joint is prismatic.
%             Otherwise, joints can be a scalar if all joints are of the
%             same type.
%    varargin = variable input arguments, representing each brench of the
%               tree configuration. See the examples below for clarity.
%               The last optional input, enclosed in curly brackets, 
%               represents which are the prismatic joints.
%
%    Returns:
%    J = the spatial Whole-Body Jacobian
%
%
%   Example 1:
%   we want to calculate the spatial Whole-Body Jacobian for the following
%   tree-configuration
%
%          |3|       |5|
%            \       /
%            |2|   |4|
%              \   /
%               |1|
%
%   The sintax is the following(P and Z have already been built)
%
%           J = spatial_WB_Jacobian(P,Z,joints,[1,2:3],[1,4:5])
%
%   where [1,2:3] and [1,4:5] are, in fact, vectors (list) containing the 
%   joints belonging to the branch.
%   The ":" between two numbers is a shortcut used instead of listing all
%   the numbers(this because all the joints are numbered in sequential
%   order)
%   
%
%   Example 2:
%   we want to calculate the  spatial Whole-Body Jacobian for the following
%   tree-configuration
%                       
%                  |6| |7|
%                    \ /
%          |3|       |5|
%            \       /
%            |2|   |4|
%              \   /
%               |1|
%
%   The sintax is the following
%
%        J = spatial_WB_Jacobian(P,Z,joints,[1,2:3],[1,4:5,6],[1,4:5,7])
%
%   
%

function J = spatial_WB_Jacobian_temp(P,Z,joints,varargin)
%
if all(size(joints)==[1,1])
   joints = joints*ones(1,size(P,2));
end
%
q0 = zeros(size(joints)); %q0 = joints ;
q = size(P,2); % joints number
if(size(q0,1) == 1 && size(q0,2) == 1)
    q0 = sign(q0)^2*ones(1,q);
else
    if(size(q0,1) == q || size(q0,2) == q)
    else
        error('q0 must be a [1 x num_joints] vector, or a single scalar');
    end
end
        
if(is_row(Z) || is_column(Z)) % the second input is only a vector, means  
    if(is_row(Z))             % that all the joints have the same joint axis
        Z = Z';
    end
    z = Z;
    Z = [];
    for i = 1 : q
        Z = [Z z];
    end
end
        
if(size(P) ~= size(Z))
    error('P and Z must be the same size');
end

n_arg = nargin;
if(iscell(varargin{nargin-3}))    % if the last input is a cell array,
    vect = varargin{nargin-3} ;    % means that is the prismatic joints configuration
    n_arg = n_arg-1;
end

%joints = zeros(1,q);

if(exist('vect','var'))
    for i = 1 : size(vect,2)
    joints(vect{i}) = 1;    % for each joint, 1 represents the prismatic joint
    end
end

conf = [];
J = [];

for i = 1 : n_arg-3 
  
    act_conf = varargin{i};
    s = size(act_conf,2);
    act_q0 = [];
    for k = 1 : s
        act_q0 = [act_q0 q0(act_conf(k))];
    end
    
    % for each serial configuration, calculates the Jacobian
    J_chain_i = singleSerialJacobian(P,Z,act_q0,act_conf,joints);

    % looks in the configuration to find how to merge the jacobians
    c = intersect(conf,act_conf);   
    if(isempty(c)) % case of two indipendent serial chains
        J = blkdiag(J,J_chain_i);
        conf = [conf act_conf];
    else  % if the global configuration has joints in common to the actual
        k = size(J,2);
        l = size(J,1);
%         J_inf = sym(zeros(6,k+s-size(c,2)));
        J_inf = zeros(6,k+s-size(c,2));
        for m = 1 : s
            J_inf(:,act_conf(m)) = J_chain_i(:,m);
        end
        
        J_vert = zeros(l,s-size(c,2));
        J = [J J_vert;
             J_inf];
        
        conf = [conf act_conf];
        conf = unique(conf);    % remove duplicates
    end
end

end
